// -*-C++-*-
//
// File:      OmImageDisplay.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Sep  6 12:10:40 MEST 2002
// Usage:     -
// Info:      -

#include "OmImageDisplay.h"
#include <cstring>
#include <fstream>
#include <sstream>
#include <cstdio>
#include "AngleFormatter.h"
#include <qstring.h>
#include "OmgSourceParameters.h"
#include "OmGSourceTempFile.h"
// Fortran prototype
extern F90MODVARDEF(OmGSourceParameters, omsource_glue, mysources);



char OmImageDisplay::_regionFile[L_TMPFILE];


/************************************************************
*
***********************************************************/
OmImageDisplay::OmImageDisplay()
{
 
    TempFile::createTempFile(_regionFile);

}
/************************************************************
*
***********************************************************/
OmImageDisplay::~OmImageDisplay()
{
  remove(_regionFile);  
 
}
/***************************************************************
* displayRegions
****************************************************************/
void OmImageDisplay::displayRegions()
{

    OmGSourceParameters& source = F90MODVAR(omsource_glue, mysources);
  
 
    if (source.ubound() < 1) return;
    send("regions deleteall");
   
    ofstream os(_regionFile);




 

    os << "global color=yellow select=0 edit=0 move=0 delete=1" << endl;
    int k = 1;
    for (int i = source.lbound(); i <= source.ubound(); i++) 
    {

          int no = source[i].spectrumNo;




      

           
          if (no > 0 && no < 999)
	  {


	       float width = 20.0; 
	       float height = 500.0;
               float y = source[no].ycoord + height / 2.0;
               // Draw the zeroth-order spectrum 
               os << " box("
	          << source[no].xcoord << ','
	          << y                 << ','
                  << width             << ','
	          << height            << ','
	          << 0.0 << ')'   
	          << " #text='" << k++ << "' color=blue" 
	          << endl;
	  
	       os << "point(" << source[no].xcoord << " " << source[no].ycoord << ") # point=cross "
	      	  << " color=black" << endl;
  	  
	      

	       continue;


	       // Draw the zeroth-order spectrum 
               os << " ellipse("
	          << source[no].xcoord << ','
	          << source[no].ycoord << ','
	          << source[no].semia  << ','
	          << source[no].semib << ','
	          << source[no].theta << ')'  // Position angel of ellipse  
	          << " #text='" << no << "' color=cyan" 
	          << endl;
	  }

	  // Draw the first-order spectrum 
          no = source[i].origin;
          if (no >= 0 && no < 9999)
	  {            
	    //	    	 os << "point(" << source[no].xcoord << " " << source[no].ycoord << ") # point=cross "
	    //  	 << " text='" << k++ << "' color=black" << endl;
  	  
	       
              os << " ellipse("
	         << source[no].xcoord << ','
	         << source[no].ycoord << ','
	         << source[no].semia  << ','
	         << source[no].semib << ','
	         << source[no].theta << ')'  // Position angel of ellipse  
	         << " #text='" << no << "' color=blue" 
	         << endl;
          
	  }

    }
   
    os.close();
    send("regions color cyan");
    send("regions file " + string(_regionFile));
   


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::toRegion(const OmSourceParameters& sources, 
                           const bool deleteRegions)
{
       
    if (deleteRegions) send("regions deleteall");
	
  

    ofstream os(_regionFile);

    os << "global color=yellow select=0 edit=0 move=0 delete=1" << endl;
  
    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {


        OmSourceParameter& s = sources[id];   

        os << "point(" << s.xcoord << " " << s.ycoord << ") # point=cross "
	   << " text='" << id << "' color=cyan" << endl;
  



   
	os << " ellipse("
	   << s.xcoord << ','
	   << s.ycoord << ','
	   << s.semia  << ','
	   << s.semib << ','
	   << s.alpha << ')'  // Position angel of ellipse  
	   << " #text='" << id << "' color=cyan" 
	   << endl;





    }
   

    os.close();
    send("regions color cyan");
    send("regions file " + string(_regionFile));
   
  
  


}



/***************************************************************
*
****************************************************************/
void OmImageDisplay::toRegion(const OmSourceParameters& sources, 
                           const vector<int>list, const bool deleteRegions)
{
    if (deleteRegions) send("regions deleteall");
	    

    ofstream os(_regionFile);

    os << "global color=yellow select=0 edit=0 move=0 delete=1" << endl;
   
    for (int id = sources.lbound(); id <= sources.ubound(); id++) 
    {

        
        OmSourceParameter& s = sources[id];   
        if (list[id - 1] > -1 && list[id - 1] < 9999)
	{
            os << "point(" << s.xcoord << " " << s.ycoord << ") # point=cross "
	       << " text='" << id << ",SP" << list[id - 1] << "' color=blue" << endl;
        }
        else
        {
            os << "point(" << s.xcoord << " " << s.ycoord << ") # point=cross "
	       << " text='" << id << "' color=green" << endl;
        

        }



	os << " ellipse("
	   << s.xcoord << ','
	   << s.ycoord << ','
	   << s.semia  << ','
	   << s.semib << ','
	   << s.alpha << ')'  // Position angel of ellipse  
	   << " #text='" << id << "' color=cyan" 
	   << endl;





    }
   

    os.close();
    send("regions color cyan");
    send("regions file " + string(_regionFile));
   


}
/***************************************************************
*
****************************************************************/
void OmImageDisplay::drawLine(int x1, int y1, int x2, int y2)
{
 
    

    ofstream os(_regionFile);

    os << "global color=yellow select=0 edit=0 move=0 delete=1" << endl;
    
    os << "line(" << x1 << " " << y1 <<  " " << x2 << " " << y2 << ") # color=red" << endl;
  
    os.close();
    send("regions color cyan");
    send("regions file " + string(_regionFile));
   
}

/************************************************************
*
***********************************************************/
void OmImageDisplay::saveRegions()
{
    char text[100];
    sprintf(text, "regions save %s", _regionFile);
    send(text);
   
}

