// -*-C++-*-
//
// File:      OmSourceBrowser.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Sep  6 11:11:03 MEST 2002
// Usage:     -
// Info:      -

#ifndef OmSourceBrowser_h
#define OmSourceBrowser_h
#include <qwidget.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qstring.h>
#include <qspinbox.h>
#include <qapplication.h>
#include "OmSourceParameters.h"
#include <cstdlib>
// forward declarations
class OmSourceParameter;
class OmSourceEditor;
class OmImageDisplay;
class OmSourceLog;
class QListBox;
class QComboBox;
class QPushButton;
class QProgressDialog;
class OmProgressDialog;

class QSpinBox;
class OmSourceGroupBox;
class QLabel;
class OmSourceButton;
class QContextMenuEvent;
class PhotometryDlg;  
class QProcess;
class QPopupMenu;
class QStringList;
class Extension;
class ExtractionRegion;
class ExtractionParameters;
class spectrumRegion;
class QImage;


class OmSourceLabel;


#include <iostream>

#define L_TMPFILE 50
class OmSourceBrowser : public QWidget
{
  Q_OBJECT
public:
  OmSourceBrowser(QWidget* parent,const char* name=0);
  virtual ~OmSourceBrowser();
  void Initialise();
  void close1(); 
  QPoint getSize() { return _pointSize ;};
private:
    static void addFileToList(QStringList &list, const string&);
    static void loadUsedFileNames(QStringList &list, const char *name);
    void paintEvent(QPaintEvent*);  
    
 
 
signals:
  // Emitted whenever we expect the user to add sources within the display
  //  void suspendedForAddition(); 
    
      void HideParent();
      void ShowParent();
      
      void SetCaption(QString caption);
      void SetStatusBarMessage(QString& caption, long time);
      void SetStatusBarMessage(QString& caption);
     
      void Exit();
      void Repaint(); 
      
 public slots:



  void displayInformation(); 
  private slots:  
  void showHelpInformation();
  void omgrismInformation();
  void omgrismPlotInformation();


  void viewAll(); // zoom out and display all sources


  void selectSpectra();

  void deleteSource(); // triggered by _editor
  void setListBoxItem(int index);
  void selectSource(int no); 
  void SetSpinBoxValue(int num);
    
     
  
  void deleteList();
  //  void detectSources();   
  void okay();
  //  void quit();

  void updateDisplay();
  void displaySource();
  void showObservationDetails();  
  void showImageDetails();  
  void restart();
  void loadImage();
  void loadImage1(int id);

  void loadSourceList();
  void loadNewSourceList(string name = " ");

  void loadSourceList1(int id);
             
  //  void contextMenuEvent( QContextMenuEvent * );
  virtual void mousePressEvent( QMouseEvent* event);
  //  void showHelpInformation();
  void showHistory();
  void showSourceTable();  
  void displayImageFileMenu();
  void displayOldSourceListFileMenu();

  void getImageFile();
  void getOldSrcListFile();
  void getOldSpectrumFile();
 
protected:
    
    public slots:
    void selectionChanged(int num) { cout << "calling setspinboxvalue i=" << num << endl;  SetSpinBoxValue(num); };
  void finishedDisplayingSpectrum() {cout << "finished displaying spectrum\n"; exit(0); };

public:
    class ObsDetails {
    public:
      QString telescope;
      QString instrument;
      QString obsId;
      QString expId;
      QString dateObs;
      QString dateEnd;
      QString filter;
      QString object;
      QString observer;
      float exposure;
      double JDStart, JDEnd, JDMiddle;
 
       
      ObsDetails() : telescope(""), instrument(""), obsId(""), 
           expId(""), dateObs(""), dateEnd(""), filter(""), 
           object(""), observer(""), exposure(0.0) 
           {};
     ~ObsDetails() {};
      ObsDetails(ObsDetails &obsDetails)
      {
	telescope = obsDetails.telescope;
        instrument = obsDetails.instrument;
        obsId = obsDetails.obsId;
        dateObs = obsDetails.dateObs;
        dateEnd = obsDetails.dateEnd;
        filter = obsDetails.filter;
        object = obsDetails.object;
        exposure = obsDetails.exposure;

      };        

    };

  class ImageDetails {
    public:
      
      bool binBpe;
      short binAx1;
      short binAx2;
      float frameTime;
      float deadFraction; 
      float background;
    int nx, ny;
      ImageDetails() :  
           binBpe(false), binAx1(1), binAx2(1),
           frameTime(0.0), deadFraction(0.0),
           background(0.0), nx(0), ny(0)   {};
     ~ImageDetails() {};
             

    };



  ObsDetails _obsDetails;
  ImageDetails _imageDetails;
  enum{OKAY, FAILED, INTERRUPTED, CHANGED, QUIT};

  // disabled member function
  static void displayObservationDetails(const OmSourceBrowser::ObsDetails *obsDetails);  
  static void displayImageDetails(const OmSourceBrowser::ImageDetails *imageDetails);  
  

  OmSourceBrowser(const OmSourceBrowser&);
  OmSourceBrowser& operator=(const OmSourceBrowser&);

  //
  void refreshListBox();

  void setSourceCoordinates(); 
 
  typedef void(OmSourceBrowser::*PRINT)(QPrinter* printer, QPainter* p,  
				       QFontMetrics *fm,  QPaintDeviceMetrics* metrics);

   PRINT Print;
  

  
   void printHeading(QPainter* p, QFontMetrics* fm, QPaintDeviceMetrics* metrics,
                     const int MARGIN, int &yPos);
   void print();
 
   bool getNewSrcListFile();
  bool getNewSpectrumFile();
  bool getNewPlotFile();

   void saveFileNames();
   void readFileNames();
  
  //  Info::Task *t;

  // member data
    OmSourceGroupBox*     _group;

    bool             _changed; 
    float _calApertureRadius;
  
    static OmImageDisplay*  _display;  
   
    vector<spectrumRegion> _spectrumList;
 
    OmSourceEditor*  _editor;
    QProgressDialog* _progress;
    OmProgressDialog* _reporter;


    QSpinBox*        _spinBoxNumber; 


 
  //    OmSourceButton*    _viewAllBtn;
    OmSourceButton*     _selectSpectraBtn;

    OmSourceButton*     _printLogBtn;
    OmSourceButton*     _printAveragesBtn;
    OmSourceButton*     _deleteAveragesBtn;

    OmSourceButton*     _sourceTableBtn;
 
    OmSourceButton*     _deleteListBtn;

  //    OmSourceButton*     _detectSourcesBtn;
    OmSourceButton*     _deleteSourceBtn;
    OmSourceButton*     _viewSourceBtn;
   

   
    OmSourceButton*     _exitBtn;
 
    OmSourceButton*     _cancelBtn;
    OmSourceButton*     _restartBtn;

    OmSourceButton*     _loadImageBtn;
    OmSourceButton*     _loadSourceListBtn;


    QLabel* _magnitudeLabel;

    OmSourceLabel *_instructionsLabel;

    QLineEdit* _xCoord;
    QLineEdit* _yCoord;
    QLineEdit* _ascension;
    QLineEdit* _declination;
   

    int _sourceNo;

    static string _imageFileName;              // For the command-line image file nam
    string _oldSourceListFile;          // For the old source-list file name
    string _newSourceListFile;          // For the new source-list file name    
    string _userNewSourceListFileName;  // For the command-line new source-list file name    
    string _userNewSpectrumFileName;    // For the command-line new source-list file name    
  
    string _oldSpectrumFile;            // For the old spectrum file
    string _newSpectrumFile;            // For the new spectrum file
    string _spectrumPlotFile;            // For the spectrum-plot file
        


    char _tempOldSourceListFile[L_TMPFILE];    // Temporary file for storing old source-list file
    char _tempNewSourceListFile[L_TMPFILE];    // Temporary file for storing old source-list file

    char _tempOldSpectrumFile[L_TMPFILE];      // Temporary file for storing new source-list file
    char _tempNewSpectrumFile[L_TMPFILE];      // Temporary file for storing old spectrum-list file
    char _tempSpectrumPlotFile[L_TMPFILE];     // Temporary file for storing new spectrum-plot file
 
  

    bool _sourceListHasChanged;
    bool _sourceListIsLoaded;     
    bool _userNewSourceListFile;
    bool _userNewSpectrumFile;

    PhotometryDlg *_photometryDlg;  
    QPopupMenu* _informationMenu;
       
  // The following are to store the image/source-list files accessed
    QStringList _imageFileList;
    QStringList _oldSourceFileList;
    QStringList _newSourceFileList;

    Extension *_extension; 
    SourceBoxList _sourceBoxList;   
    vector<int> _spectrumNoList;
    int _noSpectra;

    QProcess * _process; 
   
    ExtractionParameters* _extractionParameters;  

	   
 


    QPoint _pointSize; 

};
#endif















