#include "OmSourceDetectDialog.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"
#include <qstring.h>
#include <cstring>
#include <qspinbox.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <omdetect.h>
#include <ExceptionErrorHandler.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qdatetime.h>
#include <qprocess.h>
#include <qtextedit.h>
#include <iostream>
#include <qcombobox.h>
#include <qfont.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include "Dal.h"
#include "OmSourceButton.h"
#include "OmSourceLabel.h"
#include "OmSourceGroupBox.h"
#include "OmSourceBrowser.h"
F90MODPROCPROTO(void, omsource_glue, updatespectrumlistfile)();

/********************************************************************************
 * Dialog for running omdetect- allows you to change the parameters nsigma
 * and minsignificance.          
 *
 *******************************************************************************/

DetectionDlg::DetectionDlg(QWidget *parent, QString imageFile, 
          QString sourceListFile, QString spectrumFile) :
  QDialog(parent, "", false), _imageFile(imageFile), _sourceListFile(sourceListFile),
  _spectrumFile(spectrumFile)
{
  
    setCaption(omsourceVersionNo());
  

   // create the image buttons
    _label = new OmSourceLabel(this, "", 10, 10, 450, 50 );
  
   
    _label->setAlignment(Qt::AlignCenter);
 
    _label->setEnabled(true);
  
    _group = new OmSourceGroupBox( this, "omdetect parameters- please change if you wish",
               10, 100, 400, 65);
      
 
    _programName = "omdetect";  
    _group->setTitle("Please change the following parameter if you wish");

    int xPos = 20;
    int yPos = 20;
    int height = 25; 


    _label1 = new OmSourceLabel(_group, "nsigma", xPos,  yPos, height, 
				QString("parameter nsigma (pixel values with a value >") +
                                QString("= background mode + nsigma * sqrt(background mode)" +
                                QString("will be regarded as being part of a possible source"));


    _spinBox1 = new QSpinBox(_group, "a");

    _spinBox1->setGeometry(xPos + 100, yPos, 60, height);
 
 
    _spinBox1->setMinValue(1);
    _spinBox1->setMaxValue(20);
    _nSigma = 2;
    _spinBox1->setValue(_nSigma);
   
 
    QToolTip::add( _spinBox1, "Change the parameter nsigma (3-20");

    connect(_spinBox1, SIGNAL(valueChanged(int)), this, SLOT(  setnSigma(int)));

    _spinBox1->show();

   

     yPos += 30;
 
    QGroupBox *group = new OmSourceGroupBox( this, "", 20, 250, 200, 80);   
   
 
    QHBoxLayout* layout = new QHBoxLayout(group);
    
    layout->setMargin(10);
    layout->setSpacing(10);  
   

    _terminateBtn = new OmSourceButton(group, "Terminate");
    _terminateBtn->setDisabled(true);
    layout->addWidget( _terminateBtn);
 
   
    QObject::connect(_terminateBtn, SIGNAL(clicked()), this, SLOT(terminate()));
  

    _computeBtn = new OmSourceButton(group, "Compute");
   
    
    layout->addWidget( _computeBtn);
   
    connect(_computeBtn, SIGNAL(clicked()), this, SLOT(runOmdetect()));

  


    _omdetectComboBox = new QComboBox(false, this);
    _omdetectComboBox->setGeometry(10, 350, 600, 50);
    _omdetectComboBox->show();    



    _quitBtn = new OmSourceButton(this, "Quit", 250, 420, 120, 30);
    _quitBtn->adjustSize();
    _quitBtn->setEnabled(true);
    
   
    QObject::connect(_quitBtn, SIGNAL(clicked()), this, SLOT(quit()));
  


    _timer = new QTimer( this);
   
 
    connect( _timer, SIGNAL(timeout()), this, SLOT(updateTime()) );
 
   
   


};
/********************************************
 *
 ********************************************/
DetectionDlg::~DetectionDlg()
{
  delete _timer;


}

/*******************************************************************
* NB Need this routine to allow the widget changes to be made
*
*******************************************************************/
void DetectionDlg::runOmdetect()
{

    Info::Task *t;
    t = task("omdetect");
   

    QDateTime date = QDateTime::currentDateTime();
    _programName = "omdetect";  
    
    QString text; 
        text.sprintf("Running omdetect, version %s, %s", 
                   t->version(), (const char*)date.toString());
	//     QFont serifFont( "Times", 20, QFont::Bold );
   
	//    _label->setFont(serifFont);
    t = task("omgsource");
   
   
    _label->setText(text);
     
    _spinBox1 -> setDisabled(TRUE);
  
  

    repaint();
    qApp->processEvents();
  

     
    _process = new QProcess();
    _process->addArgument("omdetect");
    _process->addArgument("set="+_imageFile);
    _process->addArgument("outset="+_sourceListFile);
    _interupted = false;
   
    text.sprintf("nsigma=%f", (float)_nSigma);
    _process->addArgument(text);

 



   QObject::connect( _process, SIGNAL(processExited()), this, SLOT(finished()));
   QObject::connect( _process, SIGNAL(readyReadStdout()), this, SLOT(slotDataOnStdout()));
   
 
   _computeBtn->setDisabled(true); 
   _omdetectComboBox->clear();
  _quitBtn->setDisabled(true);
 
    ExceptionErrorHandler local;
    ErrorHandler* oldHandler = errHandler.handler(&local);
  
   try {
	
   if ( !_process->start())
   {   
      QMessageBox::warning(this, "omdetect", "failure", "main window", "exit", QString::null, 0, 1); 
	
   }
   else
   {
     _time.start();
     _timer->start(1000);
   }
   }
   catch (SasException::Error e) {
	QString message(e.message.c_str());
	message += "\nReturn to main window or exit program?";

	QMessageBox::warning(this, "omsource", message, "main window", "exit", QString::null, 0, 1); 
	done(OmSourceBrowser::FAILED);
      
   }
   errHandler.handler(oldHandler); // return to default error handler
  


  _terminateBtn->setEnabled(true);
  _quitBtn->setDisabled(true);

 
 
    
}

/*******************************************************************
* NB Need this routine to allow the widget changes to be made
*
*******************************************************************/
void DetectionDlg::runOmgrism()
{

    Info::Task *t;
    t = task("omgrism");
   
    QDateTime date = QDateTime::currentDateTime();
    _programName = "omgrism";  
    
    QString text; 
    text.sprintf("Running omgrism, version %s, %s", 
                   t->version(), (const char*)date.toString());

         
    _label->setText(text);
     

    repaint();
    qApp->processEvents();
  
 
    _process = new QProcess();
  

    _process->addArgument("omgrism");
  

    _process->addArgument("set="+_imageFile);
 

    _process->addArgument("sourcelistset="+_sourceListFile);
     

    _process->addArgument("outset="+_spectrumFile);


   _process->addArgument("extractfieldspectra=yes");




    _interupted = false;
   


    connect( _process, SIGNAL(processExited()), this, SLOT(terminate()));
    connect( _process, SIGNAL(readyReadStdout()), this, SLOT(slotDataOnStdout()));
    connect( _process, SIGNAL(processExited()), this, SLOT(finished1()));
  


    _computeBtn->setDisabled(true); 
    _omdetectComboBox->clear();
    _quitBtn->setDisabled(true);
 
    ExceptionErrorHandler local;
    ErrorHandler* oldHandler = errHandler.handler(&local);
   

   try {
	
   if ( !_process->start())
   {   
 

      QMessageBox::warning(this, "omgrism", "failure", "main window", "exit", QString::null, 0, 1); 
	
   }
   else
     _time.start();
   }


   catch (SasException::Error e) {
	QString message(e.message.c_str());
	message += "\nReturn to main window or exit program?";

	QMessageBox::warning(this, "omgsource", message, "main window", "exit", QString::null, 0, 1); 
	done(OmSourceBrowser::FAILED);
      
   }
   errHandler.handler(oldHandler); // return to default error handler
  


  _terminateBtn->setEnabled(true);
  _quitBtn->setDisabled(true);

 
   t = task("omgsource");
  
    
}

/**********************************************************
* Arrives here when omdetect finished running
*
********************************************************/
void DetectionDlg::finished()
{
  delete _process;
   
   _terminateBtn->setDisabled(true);
   _label->setText(" ");
   if (_interupted) return;

  
   runOmgrism();
 
}
/**********************************************************
* Arrives here when omgrism finished running
*
********************************************************/
void DetectionDlg::finished1()
{
    delete _process;
   

    QFile file(_spectrumFile);
    if (!file.exists())
    {
        QString text; 
        QDateTime date = QDateTime::currentDateTime();
  
        text.sprintf("%s, %s", (const char*)omsourceVersionNo(),  
                    (const char*)date.toString());
 
  
        
         QMessageBox::critical(0, text, "NO SPECTRA WERE PRODUCED!");  

     
  
         done(OmSourceBrowser::FAILED);
         return;
    }


    DataSet *set = dataSetServer->open(_spectrumFile, Dal::Read);
    
    int noBlocks = set->blocks();  // Check how many blocks there are in the file

    dataSetServer->close(set); 
    if (noBlocks == 0)
    {

        QDateTime date = QDateTime::currentDateTime();
  
        QString text; 
        text.sprintf("%s, %s", (const char*)omsourceVersionNo(),  
                    (const char*)date.toString());
 
   
        QMessageBox::critical(0, text, "NO SPECTRA WERE PRODUCED!");  
        
        
        done(OmSourceBrowser::FAILED);
        return; 
    }
    
    done(OmSourceBrowser::OKAY);
  
}
/***********************************************
*
**********************************************/
void DetectionDlg::terminate()
{
  _process->tryTerminate();
  _terminateBtn->setDisabled(true);
  _quitBtn->setEnabled(true);
  _computeBtn->setEnabled(true); 
 
  QTimer::singleShot(5000, _process, SLOT(kill()));

  _interupted = true;  
  _label->setText(" ");
    
    _spinBox1 -> setEnabled(TRUE);
   
   

  
    _group->setEnabled(true);
    //  _timer->stop();


}
/*******************************************************
*
******************************************************/
void DetectionDlg::slotDataOnStdout()
{
  while( _process->canReadLineStdout() ) {
   
    QString line = _process->readLineStdout();
 
    _omdetectComboBox->insertItem(line);
  
   
  }
  
}
/**************************************************************
 *
 *************************************************************/
void DetectionDlg::updateTime()
{


   QDateTime date = QDateTime::currentDateTime();


    Info::Task *t;
    t = task("omdetect");
  
    QString text; 
    text.sprintf("Running %s, version %s, %s, elapsed time=%d secs Please wait", 
        _programName.c_str(), t->version(), 
       (const char*)date.toString(), int(double(_time.elapsed()) / 1000.0));
      
   
    _label->setText(text);
 
}








