#include "OmSourceGrismDialog.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"
#include <qstring.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <omgrism.h>
#include <Dal.h>
#include <ExceptionErrorHandler.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qdatetime.h>
#include <qprocess.h>
#include <qtextedit.h>
#include <iostream>
#include <qcombobox.h>
#include <qfont.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include "OmSourceButton.h"
#include "OmSourceLabel.h"
#include  "OmSourceGroupBox.h"
#include "OmGSourceExtractionRegion.h"
#include "OmGSourceExtractionParameters.h" 
#include <cstdlib>
#include<qstrlist.h>
/********************************************************************************
 * Dialog for running omdetect- allows you to change the parameters nsigma
 * and minsignificance.          
 *
 *******************************************************************************/

OmgrismDlg::OmgrismDlg(QWidget *parent, QString imageFile, 
        QString sourceListFile, 
        QString spectrumFile, 
        QString spectrumPlotFile,
        ExtractionRegion* extractionRegion, 
        ExtractionParameters *extractionParameters) :
        QDialog(parent, "", false), 
         _imageFile(imageFile), 
         _sourceListFile(sourceListFile),
         _spectrumFile(spectrumFile),  
         _spectrumPlotFile(spectrumPlotFile),
         _extractionRegion(extractionRegion), 
	_extractionParameters(extractionParameters)
{

    setCaption(omsourceVersionNo());

     // create the image buttons
    _label = new OmSourceLabel(this, "", 10, 10, 500, 50 );
     
    _label->setAlignment(Qt::AlignCenter);
 
    _label->setEnabled(true);
 

    //    _commandLabel = new OmSourceLabel(this, "", 10, _label->geometry().bottom() + 20, 500);
     
    //    _commandLabel->setAlignment(Qt::AlignCenter);
    //    _commandLabel->setEnabled(true);
 
 
    _omgrismComboBox = new QComboBox(false, this);
    _omgrismComboBox->setGeometry(10, 80, 600, 50);
    _omgrismComboBox->show();    
    _terminateBtn = new OmSourceButton(this, "Terminate", 100, 150, 70, 30);
    _terminateBtn->setEnabled(true);
    
    connect(_terminateBtn, SIGNAL(clicked()), this, SLOT(terminate()));


    _timer = new QTimer( this);
   
 
    connect( _timer, SIGNAL(timeout()), this, SLOT(updateTime()) );
   
    move(300, 200);  
    runOmgrism();

};


/********************************************************************************
 * Dialog for running omdetect- allows you to change the parameters nsigma
 * and minsignificance.          
 *
 *******************************************************************************/

OmgrismDlg::OmgrismDlg(QWidget *parent, QString imageFile, 
        QString sourceListFile, 
        QString spectrumFile, 
        QString spectrumPlotFile, 
		       float centralWidth, 
	float leftOffset, float leftWidth,
	float rightOffset, float rightWidth,
	int binSize,
        ExtractionParameters *extractionParameters) :
        QDialog(parent, "", false), 
         _imageFile(imageFile), 
         _sourceListFile(sourceListFile),
         _spectrumFile(spectrumFile),  
	_spectrumPlotFile(spectrumPlotFile),
	_extractionParameters(extractionParameters)
{






  //  cout << "omsourcegrismdlg line 112 constructor\n";


    setCaption(omsourceVersionNo());

     // create the image buttons
    _label = new OmSourceLabel(this, "", 10, 10, 500, 50 );
     
    _label->setAlignment(Qt::AlignCenter);
 
    _label->setEnabled(true);
 

    //   cout << "omsourcegrismdlg line 125 constructor\n";


 
    _omgrismComboBox = new QComboBox(false, this);
    _omgrismComboBox->setGeometry(10, 80, 600, 50);
    _omgrismComboBox->show();    
    _terminateBtn = new OmSourceButton(this, "Terminate", 100, 150, 70, 30);
    _terminateBtn->setEnabled(true);
    
    connect(_terminateBtn, SIGNAL(clicked()), this, SLOT(terminate()));


    _timer = new QTimer( this);
   

    //   cout << "omsourcegrismdlg line 141 constructor\n";




    connect( _timer, SIGNAL(timeout()), this, SLOT(updateTime()) );
   
    move(300, 200);  
    


  
    runOmgrism(centralWidth, 
               leftOffset, leftWidth, 
               rightOffset, rightWidth, binSize);


};




/*******************************************************************
* NB Need this routine to allow the widget changes to be made
*
*******************************************************************/
void OmgrismDlg::runOmgrism()
{
    Info::Task *t;
    t = task("omgrism");
      

    //    cout << "runomgrism line 166\n";

    QDateTime date = QDateTime::currentDateTime();

  
    QString text; 
    text.sprintf("Running omgrism, version %s, %s %s", 
                   t->version(), (const char*)date.toString(), " Please wait");

    //    cout << "before running omgrism- input parameters are\n";
    //    cout << "set=" << _imageFile << endl;
    //   cout << "sourcelistset=" << _sourceListFile << endl;
    //   cout << "outset=" << _spectrumFile << endl;







    //  cout << "runomgrism line 186\n";


   
    _label->setText(text);
     

    repaint();
    qApp->processEvents();


    text = "";  
    _process = new QProcess();
    _process->addArgument("omgrism");
    text += "omgrism ";
    _process->addArgument("set="+_imageFile);

    text += "set="+_imageFile + " ";
    _process->addArgument("sourcelistset="+_sourceListFile);
    text += "sourcelistset="+_sourceListFile + " ";
    _process->addArgument("outset="+_spectrumFile);

    _process->addArgument("extractfieldspectra=yes");




    //  cout << "runomgrism line 211\n";


    //    float value = -float(_extractionRegion->_omCentralRect.width() * _extractionRegion->_binSize ) / 2.0;



 /**************************************************************************************************
 *
 * Extraction-region width NB Needs to be divided by 2- omgrism uses a half-width and unbinned pixels
 *
 ******************************************************************************************************/



    float value = -float(_extractionParameters->_extractionRegionWidth * _extractionRegion->_binSize) / 2.0;


    //  cout << "runomgrism line 229\n";


    QString argument;
    text += argument + " ";
    argument.sprintf("spectrumhalfwidth=%4.1f", max(value, -20.0f));
    //      cout << argument << endl;
 
    _process->addArgument(argument);
    text += argument + "\n";
    argument.sprintf("extractionmode=%1d\n", _extractionParameters->_extractionMode);
 
    _process->addArgument(argument);


   argument.sprintf("spectrumsmoothlength=%d", _extractionParameters->_spectrumSmoothLength);

    _process->addArgument(argument);
   



    //  cout << "runomgrism line 251\n";


    /****************************************************************************************
     * NB For OMGRISM the widths have to be in unbinned pixels and the
     * central extraction width needs to be a half-width
     ***************************************************************************************/

    //    cout << "spectrum halfwidth=" <<  _extractionRegion->_omCentralRect.width() << endl;
    //    cout << "bkgwidthleft=" << _extractionRegion->_omLeftBackgroundRect.width() << endl;
    //    cout << "bkgwidthright=" << _extractionRegion->_omRightBackgroundRect.width() << endl;
    
    //    cout << "bkgoffsetleft=" << (_extractionRegion->_omCentralRect.left() -
    //                                         _extractionRegion->_omLeftBackgroundRect.right());

    //    cout << "bkgoffsetright=" << (_extractionRegion->_omRightBackgroundRect.left() -
    //             					    _extractionRegion->_omCentralRect.right());



    /******************************************************
     *
     * Left-background width
     *
     ******************************************************/




    value = -float(_extractionParameters->_leftBkgWidth * _extractionRegion->_binSize) ;


    argument.sprintf("bkgwidthleft=%4.1f\n", max(value, -40.0f));

    //      cout << argument << endl;
 
    _process->addArgument(argument);
    text += argument + " ";

    /******************************************************
    *
    * Right-background width
    *
    ******************************************************/



//  cout << "runomgrism line 298\n";


 
    //    value = -_extractionRegion->_omRightBackgroundRect.width() * _extractionRegion->_binSize ;
    value = -float(_extractionParameters->_rightBkgWidth *_extractionRegion->_binSize);


    argument.sprintf("bkgwidthright=%4.1f", max(value, -40.0f));
    //      cout << argument << endl;
 
    _process->addArgument(argument);

    text += argument + "\n";

    //    cout << "Bin size=" <<  _extractionParameters->_binSize << endl;
    //    value = -(_extractionRegion->_omCentralRect.left() -
    //      					   _extractionRegion->_omLeftBackgroundRect.right()) * _extractionRegion->_binSize;


    /******************************************************
    *
    * Left-background offset
    *
    ******************************************************/

    value = -float(_extractionParameters->_leftBkgOffset);

 



    //  cout << "runomgrism line 330\n";



    //    cout << "_extractionRegion->_omCentralRect.left()=" << _extractionRegion->_omCentralRect.left() << endl;
    //    cout << "_extractionRegion->_omLeftBackgroundRect.right()=" << _extractionRegion->_omLeftBackgroundRect.right() << endl;
    //    cout << "binsize=" << _extractionRegion->_binSize << endl;


    argument.sprintf("bkgoffsetleft=%4.1f", value);
    //        cout << argument << endl;
  
    _process->addArgument(argument);
    text += argument + "\n";
 
 //   value = -(_extractionRegion->_omRightBackgroundRect.left() -
//      					    _extractionRegion->_omCentralRect.right()) * _extractionRegion->_binSize;

 
/******************************************************
 *
 * Right-background offset
 *
 ******************************************************/

 

    value = -float(_extractionParameters->_rightBkgOffset);

    argument.sprintf("bkgoffsetright=%4.1f", value);
    //        cout << argument << endl;

 

 
    _process->addArgument(argument);
    text += argument + "\n";
 
   //   _commandLabel->setText(text);  


   //  cout << "running omgrism spectrumfile=" << _spectrumFile << endl;
   
    connect( _process, SIGNAL(processExited()), this, SLOT(finished()));
    connect( _process, SIGNAL(readyReadStdout()), this, SLOT(slotDataOnStdout()));
 
    _omgrismComboBox->clear();
  

    ExceptionErrorHandler local;
    ErrorHandler* oldHandler = errHandler.handler(&local);
  
    try {
	
    if ( !_process->start())
    {   
        QMessageBox::warning(this, "omgrism", "failure", "main window", "exit", QString::null, 0, 1); 
	
    }
    else
        _time.start();
       _timer->start(1000);
    }
    catch (SasException::Error e) {
	QString message(e.message.c_str());
	message += "\nReturn to main window or exit program?";

	QMessageBox::warning(this, "omgsource", message, "main window", "exit", QString::null, 0, 1); 
	done(OmSourceBrowser::FAILED);
      
    }
    errHandler.handler(oldHandler); // return to default error handler
   
}
/*******************************************************************************
*
*
*******************************************************************************/
void OmgrismDlg::runOmgrism(float centralWidth, 
                            float leftOffset, float leftWidth, 
                            float rightOffset, float rightWidth, int binSize)

{
    Info::Task *t;
    t = task("omgrism");
      

    //    cout << "runomgrism line 166\n";

    QDateTime date = QDateTime::currentDateTime();

  
    QString text; 
    text.sprintf("Running omgrism, version %s, %s %s", 
                   t->version(), (const char*)date.toString(), " Please wait");

    //    cout << "before running omgrism- input parameters are\n";
    //    cout << "set=" << _imageFile << endl;
    //   cout << "sourcelistset=" << _sourceListFile << endl;
    //   cout << "outset=" << _spectrumFile << endl;







    //  cout << "runomgrism line 186\n";


   
    _label->setText(text);
     

    repaint();
    qApp->processEvents();


    text = "";  
    _process = new QProcess();
    _process->addArgument("omgrism");
    text += "omgrism ";
    _process->addArgument("set="+_imageFile);

    text += "set="+_imageFile + " ";
    _process->addArgument("sourcelistset="+_sourceListFile);
    text += "sourcelistset="+_sourceListFile + " ";
    _process->addArgument("outset="+_spectrumFile);




    //  cout << "runomgrism line 211\n";


  



 /**************************************************************************************************
 *
 * Extraction-region width NB Needs to be divided by 2- omgrism uses a half-width and unbinned pixels
 *
 ******************************************************************************************************/


    //  cout << "Line 488 centralwidth=" << centralWidth << endl;
    float value = -centralWidth * (float)binSize / 2.0;


    //  cout << "runomgrism line 229\n";


    QString argument;
    text += argument + " ";
    argument.sprintf("spectrumhalfwidth=%4.1f", max(value, -20.0f));
    //      cout << argument << endl;
 
    _process->addArgument(argument);
    text += argument + "\n";
    argument.sprintf("extractionmode=%1d\n", _extractionParameters->_extractionMode);
 
    _process->addArgument(argument);


   argument.sprintf("spectrumsmoothlength=%d", _extractionParameters->_spectrumSmoothLength);

    _process->addArgument(argument);
   



    //  cout << "runomgrism line 251\n";


    /****************************************************************************************
     * NB For OMGRISM the widths have to be in unbinned pixels and the
     * central extraction width needs to be a half-width
     ***************************************************************************************/

    //    cout << "spectrum halfwidth=" <<  _extractionRegion->_omCentralRect.width() << endl;
    //    cout << "bkgwidthleft=" << _extractionRegion->_omLeftBackgroundRect.width() << endl;
    //    cout << "bkgwidthright=" << _extractionRegion->_omRightBackgroundRect.width() << endl;
    
    //    cout << "bkgoffsetleft=" << (_extractionRegion->_omCentralRect.left() -
    //                                         _extractionRegion->_omLeftBackgroundRect.right());

    //    cout << "bkgoffsetright=" << (_extractionRegion->_omRightBackgroundRect.left() -
    //             					    _extractionRegion->_omCentralRect.right());



    /******************************************************
     *
     * Left-background width
     *
     ******************************************************/




    value = -leftWidth * (float)binSize;


    argument.sprintf("bkgwidthleft=%4.1f\n", max(value, -40.0f));

    //      cout << argument << endl;
 
    _process->addArgument(argument);
    text += argument + " ";

/******************************************************
 *
 * Right-background width
 *
 ******************************************************/



//  cout << "runomgrism line 298\n";


 
  
    value = -rightWidth * (float)binSize;


    argument.sprintf("bkgwidthright=%4.1f", max(value, -40.0f));
    //      cout << argument << endl;
 
    _process->addArgument(argument);

    text += argument + "\n";

    //    cout << "Bin size=" <<  _extractionParameters->_binSize << endl;
    //    value = -(_extractionRegion->_omCentralRect.left() -
    //      					   _extractionRegion->_omLeftBackgroundRect.right()) * _extractionRegion->_binSize;


/******************************************************
 *
 * Left-background offset
 *
 ******************************************************/

    value = -leftOffset;

 



    // cout << "runomgrism line 330\n";



    //    cout << "_extractionRegion->_omCentralRect.left()=" << _extractionRegion->_omCentralRect.left() << endl;
    //    cout << "_extractionRegion->_omLeftBackgroundRect.right()=" << _extractionRegion->_omLeftBackgroundRect.right() << endl;
    //    cout << "binsize=" << _extractionRegion->_binSize << endl;


    argument.sprintf("bkgoffsetleft=%4.1f", -leftOffset);
    //        cout << argument << endl;
  
    _process->addArgument(argument);
    text += argument + "\n";
 
 //   value = -(_extractionRegion->_omRightBackgroundRect.left() -
//      					    _extractionRegion->_omCentralRect.right()) * _extractionRegion->_binSize;

 
/******************************************************
 *
 * Right-background offset
 *
 ******************************************************/

 

    argument.sprintf("bkgoffsetright=%4.1f", -rightOffset);
    //        cout << argument << endl;

 

 
    _process->addArgument(argument);
    text += argument + "\n";
 
   //   _commandLabel->setText(text);  


  cout << "running omgrism spectrumfile=\n" << text << endl;
   
    connect( _process, SIGNAL(processExited()), this, SLOT(finished()));
    connect( _process, SIGNAL(readyReadStdout()), this, SLOT(slotDataOnStdout()));
 
    _omgrismComboBox->clear();
  

    ExceptionErrorHandler local;
    ErrorHandler* oldHandler = errHandler.handler(&local);




    //    cout << "Line 648 the list of arguments\n";  
    QStringList list = _process->arguments();
    QStringList::Iterator it = list.begin();
    while( it != list.end() ) {
      //       myProcessing( *it );
      //      cout << *it << endl; 
        ++it;
    }
    



 


  
    try {
	
    if ( !_process->start())
    {   
        QMessageBox::warning(this, "omgrism", "failure", "main window", "exit", QString::null, 0, 1); 
	
    }
    else
        _time.start();
       _timer->start(1000);
    }
    catch (SasException::Error e) {
	QString message(e.message.c_str());
	message += "\nReturn to main window or exit program?";

	QMessageBox::warning(this, "omgsource", message, "main window", "exit", QString::null, 0, 1); 
	done(OmSourceBrowser::FAILED);
      
    }
    errHandler.handler(oldHandler); // return to default error handler
   
}



/*******************************************************************
* 
*
*******************************************************************/
void OmgrismDlg::runOmgrismPlot()
{
 

  //  cout << "Running omgrism plot\n";
  //    Info::Task *t;
  //    t = task("omgrismplot");

    QDateTime date = QDateTime::currentDateTime();
  
    QString text; 
    text.sprintf("Running omgrismplot, %s %s", 
                   (const char*)date.toString(), " Please wait");

         
    _label->setText(text);
     
 

    repaint();
    qApp->processEvents();

  
    _process = new QProcess();
    _process->addArgument("omgrismplot");

 
    _process->addArgument("set="+_spectrumFile);
  

    _process->addArgument("plotfile="+_spectrumPlotFile);
    QString argument;

    argument.sprintf("plotflux=%d", _extractionParameters->_plotFlux);
        _process->addArgument(argument);
    
    argument.sprintf("binsize=%d", _extractionParameters->_binSize);
        _process->addArgument(argument);
        if (_extractionParameters->_scaleBkgPlot)
            _process->addArgument("scalebkgplot=t");
    else
        _process->addArgument("scalebkgplot=f");
  

    
    connect( _process, SIGNAL(processExited()), this, SLOT(finished1()));
    connect( _process, SIGNAL(readyReadStdout()), this, SLOT(slotDataOnStdout()));
 
    _omgrismComboBox->clear();
  
 
    ExceptionErrorHandler local;
    ErrorHandler* oldHandler = errHandler.handler(&local);
  
   try {
	
   if ( !_process->start())
   {   
      QMessageBox::warning(this, "omgrismplot", "failure", "main window", "exit", QString::null, 0, 1); 
	
   }
   else
     _time.start();
   _timer->start(1000);
   }
   catch (SasException::Error e) {
	QString message(e.message.c_str());
	message += "\nReturn to main window or exit program?";

	QMessageBox::warning(this, "omgsource", message, "main window", "exit", QString::null, 0, 1); 
	done(OmSourceBrowser::FAILED);
      
   }
   errHandler.handler(oldHandler); // return to default error handler
   
}
/**********************************************
* Called when omgrism successfully finished running
***********************************************/
void OmgrismDlg::finished()
{

    for (int i = 0; i < 10; i++)
        QApplication::beep();




    delete _process;





    // Check that a spectrum was produced


    runOmgrismPlot();
  
  
}
/**********************************************
* Called when omgrism successfully finished running
***********************************************/    


void OmgrismDlg::finished1()
{ 
    delete _process;


 
    done(OmSourceBrowser::OKAY);
  
}


/*********************************************************
*
*
*********************************************************/
void OmgrismDlg::terminate()
{
  _process->tryTerminate();
  
   done(OmSourceBrowser::FAILED);
 
}

void OmgrismDlg::slotDataOnStdout()
{
   
  while( _process->canReadLineStdout() ) {
    QString line = _process->readLineStdout();
 
    _omgrismComboBox->insertItem(line);
   
  }
 
}

/**************************************************************
 *
 *************************************************************/
void OmgrismDlg::updateTime()
{

    QDateTime date = QDateTime::currentDateTime();

    Info::Task *t;
    t = task("omgrism");
  
    QString text; 
    text.sprintf("Running omgrism, version %s, %s, elapsed time=%d secs Please wait", 
                   t->version(), (const char*)date.toString(), int(double(_time.elapsed()) / 1000.0));
	//     QFont serifFont( "Times", 20, QFont::Bold );
   
	//    _label->setFont(serifFont);
   
   
    _label->setText(text);
 
}







