// -*-C++-*-
//
// File:      OmSourceParameters.h
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Aug  9 16:49:17 MEST 2002

#ifndef OmSourceParameters_h
#define OmSourceParameters_h

#include "DopeVector.h"


#include <list>

/*****************************************************************************
 * A structure for storing the four corners of a box that surrounds
 * source spectrum
 ****************************************************************************/
struct SourceBox {
    SourceBox(float x1 = 0, float y1 = 0, float width1 = 0, float height1 = 0, float angle1 = 0) :
      x(x1), y(y1), width(width1), height(height1), angle(angle1) {}
  float x, y;
  float width, height;
  float angle;
};
typedef vector<SourceBox> SourceBoxList;  // A list of source boxes






// equivalent of f90::type(OmSourceParameter)
struct OmSourceParameter {
    OmSourceParameter() :
      //    box,
    rightascension(0), 
    declination(0),  
    xcoord(0), 
    ycoord(0),
    positionerror(0),
    exposure(0), 
    significance(0),
    rawcountrate(0), 
    rawcountrateerror(0),
    corrcountrate(0),
    corrcountrateerror(0),  //
    magnitude(0),
    magnitudeerror(0),  
    rawbackgroundcountrate(0),
    rawbackgroundcountrateerror(0),
    corrbackgroundcountrate(0),
    corrbackgroundcountrateerror(0),
    psf1correction(0),
    psf2correction(0),
    coincidencecorrection(0), 
    semia(0),
    semib(0),
    theta(0),  
    photometryradius(0),   // was 0
    axialRatio(1.0),
    alpha(0.0), 
    innerbackgroundradius(0),
    outerbackgroundradius(0),
    number(0), 
    nosourcepixels(0),
    qualityflag(0),
    iMin(0), iMax(0), jMin(0), jMax(0),
    extensionflag(0),
    confusionflag(0) 

  {}   
  //    struct Box box;
    double rightascension;
    double declination;
    float xcoord; 
    float ycoord;
    float positionerror;
    float exposure; 
    float significance;
    float rawcountrate; 
    float rawcountrateerror;
    float corrcountrate;
    float corrcountrateerror;  //
    float magnitude;
    float magnitudeerror;  
    float rawbackgroundcountrate;
    float rawbackgroundcountrateerror;
    float corrbackgroundcountrate;
    float corrbackgroundcountrateerror;
    float psf1correction;
    float psf2correction;
    float coincidencecorrection; 
    float semia;
    float semib;
    float theta;  
    float photometryradius;
    float axialRatio;
    float alpha;   
    float innerbackgroundradius;
    float outerbackgroundradius;
    int number; 
    int nosourcepixels; 
    Integer2 qualityflag;
    Integer2 iMin; 
    Integer2 iMax; 
    Integer2 jMin; 
    Integer2 jMax; 
    Integer1 extensionflag;
    Integer1 confusionflag;

   };



/*****************************************************
 The observation details
*****************************************************/
struct PhotometryParameters {

  PhotometryParameters() : 
    apertureRadius(0),  
  calApertureRadius(0) 
      
  {
  }
    float apertureRadius;
    float calApertureRadius;
};
class OmSourceParameters;
F90MODPROCDEF(void,omsourceparameter_mod,new_)(OmSourceParameters*,const int&, const int&);
F90MODPROCDEF(void,omsourceparameter_mod,delete_)(OmSourceParameters*);


class OmSourceParameters : public DopeVector<OmSourceParameter>
{
public:
  // f90:: allocate(OmSourceParameter(length))
  OmSourceParameters(int length)
  {
    int lbound=1;
    int ubound=length;
    F90MODPROC(omsourceparameter_mod, new_)(this, lbound, ubound);
  }

  // f90:: allocate(OmSourceParameter(lbound:ubound))
  OmSourceParameters(int lbound,int ubound)
  {
    F90MODPROC(omsourceparameter_mod,new_)(this,lbound,ubound);
  }
  
  // f90:: deallocate(OmSourceParameter)
  ~OmSourceParameters()
  {
    F90MODPROC(omsourceparameter_mod,delete_)(this);
  }
private:
   // disabled member functions
  OmSourceParameters();
  OmSourceParameters(const OmSourceParameters&);
  OmSourceParameters& operator=(const OmSourceParameters&);
};







#endif
