#ifndef DS9SELECTDLG_H
#define DS9SELECTDLG_H



#include <qdialog.h>


class QLineEdit;
class QLabel;
class QTable;
class QSpinBox;
class QPainter;
class QCheckBox;
class OmSourceButton;
class OmSourceLabel;
class QRect;
class QPoint;
class ImageView; // Displays the image, etc
class QTimer;
class ImageDlg;
class ExtractionRegion;
class QPixmap;
class OmSourceEdit;
class QButtonGroup;
class ExtractionParameters;
class spectrumRegion;
class OmSourceBrowser;




#include <vector>


/****************************************
*
***************************************/
class spectrumRegion
{
    public:        
        float _x0, _y0;
        float _centralWidth, _height;
        float _leftOffset, _leftWidth;
        float _rightOffset, _rightWidth;
	float _xFirst, _yFirst;
    
    public:
        spectrumRegion(
	    float x0, float y0,
            float centralWidth, float height,
            float leftOffset, float leftWidth,
            float rightOffset, float rightWidth) : 
	    _x0(x0), _y0(y0),
	   _centralWidth(centralWidth), _height(height),
           _leftOffset(leftOffset), _leftWidth(leftWidth),
           _rightOffset(rightOffset), _rightWidth(rightWidth), _xFirst(0.0), _yFirst(0.0) {};

         spectrumRegion(
	    float x0, float y0,
            float centralWidth, float height,
            float leftOffset, float leftWidth,
            float rightOffset, float rightWidth, float xFirst, float yFirst) : 
	    _x0(x0), _y0(y0),
	   _centralWidth(centralWidth), _height(height),
           _leftOffset(leftOffset), _leftWidth(leftWidth),
           _rightOffset(rightOffset), _rightWidth(rightWidth), _xFirst(xFirst), _yFirst(yFirst) {};




};





/********************************************************
*
*
*
********************************************************/
class ds9SelectDlg : public QDialog
{
    Q_OBJECT
      signals:
  

    //    void displayInformation1();
  
   
public:
    ds9SelectDlg( QWidget *parent, 
	     vector<spectrumRegion> *oldSpectrumList,           
             int nx, int ny, int binY, const char * filter, 
             ExtractionParameters *extractionParameters, const char *name=0 );

    ~ds9SelectDlg();    
    int getNoSpectra() {return _noSpectra;};

  static void displayOldSpectrumRegions(vector<spectrumRegion> &list);



public slots:
    void keyPressEvent( QKeyEvent *e);
    virtual void mousePressEvent( QMouseEvent* event);


 private slots:
   
    void deleteList();
    void deleteCurrent();
    void deleteSpectrum(int no);    
    void displayCurrent();    
    void quit();    
    void okay();
    //    void displayInformation();  
  
    void selectionHasChanged(int);        
    void selectionHasChanged();        
 
    void leftBackgroundOffsetHasChanged(int);
    void leftBackgroundWidthHasChanged(int);
    void rightBackgroundOffsetHasChanged(int);
    void rightBackgroundWidthHasChanged(int);

    void widthHasChanged(int);
    void showHelpInformation();    
    

    void computeSpectrum();
 
    void getUserSpectrum();
    void regionSelectionChanged( int );
   
  

    void extractionWidthEntered(void);
    void xZeroOrderEntered(void);
    void yZeroOrderEntered(void);
    void leftBackgroundOffsetEntered(void);
    void rightBackgroundOffsetEntered(void);
    void leftBackgroundWidthEntered(void);
    void rightBackgroundWidthEntered(void);
    void plotNewSpectrum(void);
    void changeExtractionParameters();
    void zoomOut();
    void zoom();
    void zoom(int factor);
    void alignSpectrum();
    void pixelShiftChanged(int value);
    void zoomFactorChanged(int value);
    void displaySpectrum( int no );
    void centroidSpectrum();
    void startDisplay();

private:


    bool validateNumber( QLineEdit *edit, double xMin, double xMax, float &number);
    void validateAllInput();
    bool _userInput;
    bool _centralRegionSelected;
    bool _leftBackgroundRegionSelected;
    bool _rightBackgroundRegionSelected;
    bool _changed;
    bool _zoomIn;
  
   

    /*************************************************************************
    *
    *
    *************************************************************************/
    static void extractSpectrumCoordinates(float &xZero, float &yZero, 
					   float &xFirst, float &yFirst, float&width, 
					  float &leftOffset, float &rightOffset,
					   float &leftWidth, float &rightWidth);


    void setImageCounts(); 
    void initialiseDisplay();


    void displayNewSpectrumRegions();

    void displayAllSpectrumRegions();



    void enableWidgets();
    void disableWidgets();
 
   
    void viewSpectrum();    
   
    float _centralX0, _centralY0, _centralWidth;
    float _leftBackgroundX0, _leftBackgroundY0, _leftBackgroundWidth;
    float _rightBackgroundX0, _rightBackgroundY0, _rightBackgroundWidth;
    float _leftBackgroundOffset, _rightBackgroundOffset;
    float _minOffset, _maxOffset;    
    float _height;
    vector<spectrumRegion> _newSpectrumList;
    vector<spectrumRegion> *_oldSpectrumList;


    bool _userSpectrumSelected;
    int _zoomFactor;    
    int _currentNo;      
public:
    void updateTable();
  
   
    void getMinMax();
    void loadImage(); 
  
 
    bool runOmgrism();

  

    vector<ExtractionRegion>  _newExtractionRegionList;
    
    int _nx, _ny;
    int _binX, _binY;
    
    ExtractionParameters* _extractionParameters;  
  
 
     QRect _rect;
    QRect _leftRect;
    QRect _rightRect;
   
  

    int _sourceNo;
  
    QPoint _point;
   

    QRect _omCentralRect;
    QRect _omLeftBackgroundRect;
    QRect _omRightBackgroundRect;

   
    OmSourceEdit *_omUserEdit[7];



    
    int _noSpectra;
    int _regionNo;   
    int _regionStartSize;  
  
    int _ninX, _ninY;
    float _minX, _maxX;
    float _minY, _maxY;
    double _pixelShift;    
   
   
    // OmSourceBrowser::ObsDetails* obsDetails,  
    QTable* _table;
    OmSourceLabel* _coordLabel;
    OmSourceLabel* _noSourcesLabel;
    OmSourceLabel* _noLabel;
 
    OmSourceLabel* _instructionsLabel;
    OmSourceLabel* _alignLabel;

    OmSourceLabel* _imageCountsLabel;

                 
    QButtonGroup  *_bgroup; 
    OmSourceButton *_deleteListBtn;
    OmSourceButton *_deleteBtn;
    OmSourceButton *_displayBtn;


    OmSourceButton *_computeSpectrumBtn;

    OmSourceButton *_alignSpectrumBtn;

    OmSourceButton *_centroidSpectrumBtn;


    OmSourceButton *_updateRegionsBtn;
    OmSourceButton *_zoomButton;


    // Widgets for the spectrum extraction region
    QLabel* _widthLabel; 
    QSpinBox* _widthSpinBox;
  


    // Widgets for the background extraction region
    OmSourceLabel* _leftBackgroundOffsetLabel;   // Distance from left-hand edge of extraction region
    OmSourceLabel* _leftBackgroundWidthLabel;
    
    QSpinBox* _leftBackgroundOffsetSpinBox;
    QSpinBox* _leftBackgroundWidthSpinBox;
   
   
    OmSourceLabel* _rightBackgroundOffsetLabel; // Distance from right-hand edge of extraction region
    OmSourceLabel* _rightBackgroundWidthLabel;
    
    QSpinBox* _rightBackgroundOffsetSpinBox;
    QSpinBox* _rightBackgroundWidthSpinBox;
    QSpinBox* _alignSpinBox;
    QSpinBox* _zoomSpinBox;




    QTimer* _timer;
   
 
           
         
};
#endif





