! -*-f90-*-
!
! Author:  C. Brindle
! Generated: 

module omsource_glue
   use param
   use sla
   use omsource_old
   use omgsourceparameter_mod
   use omsourceparam_mod

   use omsourceregion_mod

   use omprogress_mod

   use omskybackground_mod
   use omphotometry_mod    ! In mssllib for doing aperture photometry
   use ominterpolate_mod ! In mssllib for doing 2-d bilinear interpolation 
   use copytable_mod
   implicit none
   
   type(GSourceParameters), pointer :: mySources(:) => null()              ! The global source-list
   type(GSourceParameters), pointer :: myOldSources(:) => null()           ! The global source-list

                                             
   ! The list of newly selected sources in addsourcesdlg                       
   type(GSourceParameters), pointer :: myNewSources(:) => null()           ! The global source-list
   ! A temporary list used by omgrism
   type(GSourceParameters), pointer :: myTempSources(:) => null()           ! The global source-list
  
   
   type(PhotometryParameters) :: myPhotometryParameters

contains

   ! Info: This is always the first function that is called by the GUI;
   !       in here we initialise/load/whatever




subroutine init(loaded)
    logical, intent(out) :: loaded
    
!    write(*, *) "omsourceglue line 46 myspectrumno=", mySpectrumNo
    call omsource_init(mySources, mySpectrumNo) 
    loaded = associated(mySources)
end subroutine init

 


   ! Info: If the GUI closes, it calls
 
subroutine finish()
     
    call omsource_finish(mySources)
 
end subroutine finish


subroutine loadimage(image)
real(single), pointer :: image(:, :)

    image=>myImage
 
end subroutine loadimage


! Subroutine for loading the sources from the source-list file
! produced by omdetect in omsourcebrower::SourceDetection
subroutine loadsources()
    mySpectrumNo = 0

    call omsource_loadsources(mySources, stringParameter("oldsrclistset"), mySpectrumNo)


end subroutine loadsources


! Info: Remove a single source at location id
subroutine remove(id)
    integer, intent(in) :: id
      
    call omsource_delete(mySources, id)
end subroutine remove

! Info: deletes the whole source list



subroutine deleteNewList()
    call omsource_deleteList(myNewSources)
end subroutine deleteNewList



subroutine deleteList()
    call omsource_deleteList(mySources)
end subroutine deleteList


!**********************************************
! Replaces the new list with the old one
!*********************************************
subroutine restoreOldList()

    
    if (associated(mySources) .and. associated(myOldSources)) then
       

        call delete(mySources) 
        call new(mySources, low = 1, high = size(myOldSources))
 
      
        mySources = myOldSources

    end if
    
end subroutine restoreOldList

!**********************************************
! Replaces the new list with the old one
!*********************************************
subroutine saveOldList()

    if (associated(myOldSources)) then
        call delete(myOldSources)
    end if

    if (associated(mySources)) then
  
        call new(myOldSources, low = 1, high = size(mySources))

        myOldSources = mySources
     
 
    end if
    
   
end subroutine saveOldList

!********************************************
! Replaces a source
!*******************************************
subroutine replace(id, source)
  
    integer, intent(in) :: id
    type(GSourceParameters), intent(in) :: source
        
    mySources(id) = source

end subroutine replace    

!***********************************************
!
!********************************************** 

subroutine computebackground(value, sigma)
    real(single), intent(out) :: value, sigma

   value = backgroundMode(myImage, int(size(myImage, 1), int16), &
      int(size(myImage, 2), int16), 2.0, sigma) 
end subroutine computebackground



!**************************************************
!
!
!*************************************************
real(single) Function  TwoDInterpolate(x, y)
    real(single), intent(in) :: x, y
    
    TwoDInterpolate = BilinearInterpolation1(myImage, x, y)

end Function TwoDInterpolate

!**************************************************
!
!
!*************************************************
subroutine SaveImageSection(il, iu, jl, ju)
    integer, intent(in) :: il, iu, jl, ju
    real(single), pointer :: image(:, :) => null()
    integer(int16) :: i1, i2, j1, j2, nx, ny

    i1 = max(0, il)
    i2 = min(size(myimage, 1) - 1, iu)
    j1 = max(0, jl)
    j2 = min(size(myimage, 2) - 1, ju)
  
    nx = i2 - i1 + 1
    ny = j2 - j1 + 1
    allocate(image(0 : nx - 1, 0 : ny - 1))
    image = myImage(i1 : i2, j1 : j2)

!    call SaveRealImage(image, nx, ny, trim(stringParameter("wdxset")), "Spectrum image")
    deallocate(image)
end subroutine SaveImageSection
 
!****************************************************
! This is called from addsourcesdlg whenever
! the user has selected a region for spectrum extraction
! 
!
!****************************************************
subroutine CreateGrismSourceList(n, yZero, xFirst, x1, x2, y1, y2, centroidyZero, centroidxFirst)
    integer, intent(in) :: n
    real(single) :: yZero(1 : n), xFirst(1 : n)
    real(single) :: x1(1 : n), x2(1 : n)
    real(single) :: y1(1 : n), y2(1 : n)
    integer(int8) :: centroidyZero, centroidxFirst


   
    call omsource_CreateGrismSourceList(n, yZero, xFirst, x1, x2, y1, y2, &
           mySources, myNewSources, myTempSources, mySpectrumNo, centroidyZero, centroidxFirst)




end subroutine CreateGrismSourceList



!****************************************************
! This is called from addsourcesdlg whenever
! the user has selected a region for spectrum extraction
! 
!
!****************************************************
subroutine CreateGrismSourceList1(xZero, yZero)
   
    real(single) :: xZero, yZero


   
    call omsource_CreateGrismSourceList1(xZero, yZero, &
           mySources, myNewSources, myTempSources, mySpectrumNo)




end subroutine CreateGrismSourceList1



!**********************************************************************
!
!**********************************************************************
real(single) Function GetTheBackgroundMode()

    GetTheBackgroundMode = omsource_GetBackgroundMode()

end Function GetTheBackgroundMode

!************************************************************
!
! Subroutine updatespectrumlistfiles
! Updates the old source-list and spectrum files
! by appending the data in the new source-list and
! spectrum files to them
!
!*************************************************************
subroutine UpdateSpectrumListFile
  
    call omsource_UpdateSpectrumListFile(myTempSources)

 
    call delete(myTempSources)
    nullify(myTempSources)


end subroutine UpdateSpectrumListFile



!**********************************************************
! Delete the 2 sources associated with the newly
! selecet spectrum
!*********************************************************
subroutine deletenewspectrum()


    call omsource_delete(myNewsources, size(myNewSources))
    call omsource_delete(myNewsources, size(myNewSources))

end subroutine deletenewspectrum  






!************************************************************
!
! Subroutine DeleteBlockFromSpectrumFile
! Deletes the given rate-b lock from the spectrum file
! for new sources
!*************************************************************
subroutine DeleteBlockFromSpectrumFile1(no, offset)
    integer, intent(in) :: no, offset 
    integer :: sourceNo
         
 
    character(100) :: blockName 
    type(DataSetT) :: oldSpectrumSet, newSpectrumSet
    integer :: i, j, number
    character(4) :: rate
    type(TableT) :: tab1, tab2
    type(ColumnT) :: column1
   
    integer :: noBlocks 
    
    if (.not. setExists(trim(stringParameter("oldspectrumset")))) then
        
        ! The spectrum list file created by omgrism
        call fatal("No file", "The old spectrum file doesn't exist")
         
    end if   

    sourceNo = 2 * (no - offset) + 1


    oldSpectrumSet = dataSet(trim(stringParameter("oldspectrumset")), MODIFY)
 
    noBlocks = numberOfBlocks(oldSpectrumset)

!write(*, *) "omsourceglue line 331 deleteblockfromspectrumfile1 no=", no, " sourceno=", sourceNo

    if (noBlocks == 1) then




!       write(*, *) "omsourceglue line 338 deleteblockfromspectrumfile1"

        call deleteBlock(oldSpectrumSet, 0)

        call release(oldSpectrumSet)
     
        mySpectrumNo = mySpectrumNo - 1



!write(*, *) "omsourceglue line 348 deleteblockfromspectrumfile1"




        ! Delete the new sources
        call omsource_deleteList(myNewSources)      
  
        return
    end if 






!write(*, *) "omsourceglue line 364 deleteblockfromspectrumfile1"


    if (no .eq. noBlocks - 1) then
        
      
        ! Need to delete the sources associated with this spectrum




!write(*, *) "omsourceglue line 375 deleteblockfromspectrumfile1"



        blockName = trim(name(block(oldspectrumset, no)))  
        read(blockName, '(a4, i4)') rate, number

        call deleteBlock(oldSpectrumSet, no)
        call release(oldSpectrumSet)
   


!write(*, *) "omsourceglue line 387 deleteblockfromspectrumfile1"



        call omsource_delete(myNewSources, sourceNo)
        call omsource_delete(myNewSources, sourceNo)           
 
!write(*, *) "omsourceglue line 394 deleteblockfromspectrumfile1"


        mySpectrumNo = mySpectrumNo - 1
        return 
    end if



    blockName = trim(name(block(oldspectrumset, no)))  
    read(blockName, '(a4, i4)') rate, number
   

    ! Copy old spectrum file to new one 
    call copydataSet(trim(stringParameter("oldspectrumset")), trim(stringParameter("newspectrumset")))
    newSpectrumSet = dataSet(trim(stringParameter("newspectrumset")), MODIFY)

   


!write(*, *) "omsourceglue line 414 deleteblockfromspectrumfile1"



    ! Delete the blocks in the  
    do i = noBlocks - 1, no, -1
        call deleteBlock(oldSpectrumSet, i)
    end do

  

    ! Loop through the other blocks
    do i = no + 1, numberOfBlocks(newSpectrumSet) - 1
        
        tab1 = Table(newSpectrumSet, i)
     
        tab2 = addTable(oldSpectrumSet, name(Table(newSpectrumSet, i - 1)), numberOfRows(tab1))
  
  
  
        call copyAllAttributes(attributable(tab1), attributable(tab2))

        do j = 0, numberOfColumns(tab1) - 1
            column1 = Column(tab1, j)

            call copyColumn(tab2, column1, name(column1))

        end do
      
      
    end do

    
    call release(newSpectrumSet)
    call release(oldSpectrumSet)

 
    mySpectrumNo = mySpectrumNo - 1

    ! Need to delete the sources associated with this spectrum







!write(*, *) "omsourceglue line 461 deleteblockfromspectrumfile1"


  
    ! Renumber the sources spectrum numbers & origins - decrease values by 2
    do i = sourceNo + 2, size(myNewSources), 2
      
        myNewSources(i)%spectrumNo = myNewSources(i - 2)%spectrumNo
        myNewSources(i)%origin = myNewSources(i - 2)%origin
    end do





!write(*, *) "omsourceglue line 476 deleteblockfromspectrumfile1"



    call omsource_delete(myNewSources, sourceNo)
    call omsource_delete(myNewSources, sourceNo)        
    

!write(*, *) "omsourceglue line 484 deleteblockfromspectrumfile1"



end subroutine DeleteBlockFromSpectrumFile1



!************************************************************
!
! Subroutine DeleteBlockFromSpectrumFile
! Deletes the given rate-b lock from the spectrum file
!
!*************************************************************
subroutine DeleteBlockFromSpectrumFile2(no)
    integer, intent(in) :: no 
        
    type(GSourceParameters), pointer :: tempSources(:) => null()       

    character(100) :: blockName 
    type(DataSetT) :: oldSpectrumSet, newSpectrumSet
    integer :: i, j, number(2)
    character(4) :: rate
    type(TableT) :: tab1, tab2
    type(ColumnT) :: column1
   
    integer :: noBlocks 


!write(*, *) "Deleteblockfromspectrumfile2 line 512 no=", no
   
    if (.not. setExists(trim(stringParameter("oldspectrumset")))) then
        
        ! The spectrum list file created by omgrism
        call fatal("No file", "The old spectrum file doesn't exist")
         
    end if   

    oldSpectrumSet = dataSet(trim(stringParameter("oldspectrumset")), MODIFY)
 
    noBlocks = numberOfBlocks(oldSpectrumset)




!write(*, *) "Deleteblockfromspectrumfile2 line 528 no=", no
   



    if (noBlocks == 1) then
       
        call deleteBlock(oldSpectrumSet, 0)

        call release(oldSpectrumSet)
    
 
        mySpectrumNo = mySpectrumNo - 1

        ! Delete the new sources


        call omsource_deleteList(mySources)      
  
        return
    end if 





!write(*, *) "Deleteblockfromspectrumfile2 line 554 no=", no
   


    if (no .eq. noBlocks - 1) then
      
  
        ! Need to delete the sources associated with this spectrum

        blockName = trim(name(block(oldspectrumset, no)))  
        read(blockName, '(a4, i4)') rate, number(1)
        
        number(2) = mySources(number(1))%origin
        if (number(2) < 0 .or. number(2) > 999) number(2) = number(1)

        call deleteBlock(oldSpectrumSet, no)
        call release(oldSpectrumSet)
      
        
        call omsource_delete(mySources, maxval(number))
        if (number(2) .ne. number(1)) then
            call omsource_delete(mySources, minval(number))        
        end if        
  

        mySpectrumNo = mySpectrumNo - 1
        return 
    end if




!write(*, *) "Deleteblockfromspectrumfile2 line 586no=", no
   



    call new(tempSources, low = 1, high = size(mySources))
    do i = 1, size(mySources)
        tempSources(i) = mySources(i)
    end do



!write(*, *) "Deleteblockfromspectrumfile2 line 598 no=", no
   






    blockName = trim(name(block(oldspectrumset, no)))  
    read(blockName, '(a4, i4)') rate, number(1)
   

    ! Copy old spectrum file to new one 
    call copydataSet(trim(stringParameter("oldspectrumset")), trim(stringParameter("newspectrumset")))
    newSpectrumSet = dataSet(trim(stringParameter("newspectrumset")), MODIFY)

  

    ! Delete the blocks in the  
    do i = noBlocks - 1, no, -1
        call deleteBlock(oldSpectrumSet, i)
    end do

  
    ! Loop through the other blocks
    do i = no + 1, numberOfBlocks(newSpectrumSet) - 1
        
        tab1 = Table(newSpectrumSet, i)
     
        tab2 = addTable(oldSpectrumSet, name(Table(newSpectrumSet, i - 1)), numberOfRows(tab1))
  
  
  
        call copyAllAttributes(attributable(tab1), attributable(tab2))

        do j = 0, numberOfColumns(tab1) - 1
            column1 = Column(tab1, j)

            call copyColumn(tab2, column1, name(column1))

        end do
      
      
    end do

    
    call release(newSpectrumSet)
    call release(oldSpectrumSet)




!write(*, *) "Deleteblockfromspectrumfile2 line 650 no=", no
   


    mySpectrumNo = mySpectrumNo - 1

    ! Need to delete the sources associated with this spectrum


    number(2) = mySources(number(1))%origin
    if (number(2) > 0 .and. number(2) < 999) then
        call omsource_delete(mySources, maxval(number))        
        call omsource_delete(mySources, minval(number))
    else
        number(2) = number(1)
        call omsource_delete(mySources, number(1))
    end if




!write(*, *) "Deleteblockfromspectrumfile2 line 671 no=", no
   



    if (number(1) .ne. number(2)) then  
        ! Renumber the sources spectrum numbers & origins - decrease values by 2
        do i = max(1, min(number(1), number(2))), size(mySources) - 1
       
            mySources(i)%spectrumNo =  tempSources(i + 2)%spectrumNo - 2
            mySources(i)%origin =  tempSources(i + 2)%origin - 2
       
        
        end do
    else
        do i = number(1), size(mySources)   
            mySources(i)%spectrumNo =  tempSources(i + 1)%spectrumNo - 1
            mySources(i)%origin =  tempSources(i + 1)%origin - 1        
        end do
    end if


    call delete(tempSources)


end subroutine DeleteBlockFromSpectrumFile2
!************************************************
!
!***********************************************
subroutine CreateSpectrumFile(blockNumber)
    integer, intent(in) :: blockNumber
  
    type(DataSetT) :: oldSpectrumSet
    type(DataSetT) :: newSpectrumSet
    type(BlockT)   :: blk

   
   
    ! The old spectrum list file
    oldSpectrumSet = dataSet(trim(stringParameter("oldspectrumset")), MODIFY)
    

    ! The spectrum list file created by omgrism
    newSpectrumSet = dataSet(trim(stringParameter("newspectrumset")), create)
    
    call copyAllAttributes(attributable(oldSpectrumSet), attributable(newSpectrumSet))

    blk = block( oldSpectrumSet, blockNumber, MODIFY )
    call copyBlock( newSpectrumSet, blk )


    call release(oldSpectrumSet)
    call release(newSpectrumSet)

end subroutine CreateSpectrumFile

!****************************************************
!
!
!***************************************************
subroutine CentroidSource(xl, xr, yl, yu, xPos, yPos)
    integer, intent(in) :: xl, xr, yl, yu
    real(single) :: xPos, yPos


    call omsource_CentroidSource(xl, xr, yl, yu, &
                 xPos, yPos)


end subroutine CentroidSource


!*****************************************************
! Adds the new sources to the global one
!****************************************************
subroutine UpdateSourceList()
    
    type(GSourceParameters), pointer :: tempSources(:)
    integer(int32) :: k

!write(*, *) "omsource_glue.f90 line  659 updatesourcelist"
    if (.not. associated(myNewSources)) return; 


    if (.not. associated(mySources)) then
        mySources => myNewSources


!write(*, *) "omsource_glue.f90 line  667 updatesourcelist"



        nullify(myNewSources)
        return
    end if

    k = size(mySources)
 

!write(*, *) "omsource_glue.f90 line  673 k=", k
    


    call new(tempSources, low = 1, high = k)
 
    tempSources = mySources
    call delete(mySources)
      
    call new(mySources, low = 1, high = k + size(myNewSources))
 
    mySources(1 : k) = tempSources
    call delete(tempSources)
    
    mySources(k + 1 : k + size(myNewSources)) = myNewSources

    call delete(myNewSources)
    

end subroutine UpdateSourceList
!******************************************************
!
!*****************************************************
subroutine ModifySourceListFile()

    type(DataSetT) :: newSourceListSet
    type (TableT) :: srcListTable

    newSourceListSet = dataSet(trim(stringParameter("newsrclistset")), MODIFY)
    srcListTable = Table(newSourceListSet, "SRCLIST")


    call setAttribute(srcListTable, "FILENAME", trim(stringParameter("newspectrumset")), "The spectrum set file")
    
   call release(newSourceListSet)

end subroutine ModifySourceListFile


!*******************************************
! Function ComputeEpoch
!******************************************
real(double) function  ComputeEpoch(dateObs)
   
    character(19), intent(in)  :: dateObs               
    real(double)  :: JulianDate0, JulianDate
    character(19)  :: missionReferenceTime = "1998-01-01T00:00:00"
    
    ! Compute the Julian date of the mission reference time.
    JulianDate0 = ModifiedJulianDate(missionReferenceTime)

    ! Compute the Julian date of the observation
    JulianDate = ModifiedJulianDate(dateObs)

    
    ComputeEpoch = 1998.0_double + (JulianDate - JulianDate0) / 365.25_double

end function ComputeEpoch


!*************************************************************
!
!*************************************************************
subroutine ComputeJulianDates(JDStart, JDEnd, JDMiddle)
    real(double), intent(out) :: JDStart, JDEnd, JDMiddle
    type(DataSetT) :: dataSetH
    character(20) :: string
    
    dataSetH = DataSet(trim(stringParameter("imageset")), read)


    string = trim(stringAttribute(dataSetH, "DATE-OBS"))

    JDStart = ModifiedJulianDate(string)

    string = trim(stringAttribute(dataSetH, "DATE-END"))

    JDEnd = ModifiedJulianDate(string)

 

    call release(dataSetH)


    JDMiddle = (JDStart + JDEnd) / 2.0_double

end subroutine ComputeJulianDates





end module omsource_glue














