! -*-f90-*-
!
! Author:    jbakker
! Generated: Mon Sep  9 15:37:20 CEST 2002
! Usage:     C++/f95 interface
! Info:      The new/delete methods are mainly here because
!            we want to be able to allocate FORTRAN memory from within C++
! Example:
!   type(OmSourceRegion), pointer :: myRegions(:)
!
!   call new(myRegions,low=0,high=5) ! allocate(s(0:5)
!   :
!   call delete(myRegions)
module omsourceregion_mod
   use types
   use omsourceutils_mod, only : assertOn
   implicit none

   private  ! all, but
   public  :: OmSourceRegion
   public  :: construct
   public  :: destruct
   public  :: new
   public  :: delete
   public :: list
   character(*), parameter :: myModule="omsourceregion_mod"

   type OmSourceRegion
      real(single) :: x
      real(single) :: y
   end type OmSourceRegion

   interface new
      module procedure new_
   end interface

   interface delete
      module procedure delete_
   end interface

   interface construct
      module procedure construct_
   end interface

   interface destruct
      module procedure destruct_
   end interface

 interface list
      module procedure list_
   end interface

contains

   subroutine construct_(region)
      type(OmSourceRegion),intent(out)   :: region
    
      region%x=0.
      region%y=0.
   end subroutine construct_



   subroutine destruct_(region)
      type(OmSourceRegion),intent(inout) :: region
      region%x=0.
      region%y=0.
   end subroutine destruct_



   subroutine new_(regions,low,high)
      type(OmSourceRegion),pointer    :: regions(:)
      integer,             intent(in) :: low,high

      integer                         :: i, status
 
      allocate(regions(low:high),stat=status)
      call assertOn(status/=0,myModule//".new.regions")
      do i=low,high
         call construct(regions(i))
       
      end do
   end subroutine new_

   subroutine delete_(regions)
      type(OmSourceRegion),pointer :: regions(:)
      if (associated(regions)) then
         deallocate(regions)
         nullify(regions)
      end if
   end subroutine delete_

   subroutine list_(regions)
      type(OmSourceRegion), pointer    :: regions(:)
  

      integer :: i
      write(*, *) "Listing regions in  omsourceregion_mod"


      do i = 1, size(regions)
          write(*, *) "Region ", i, " x=", regions(i)%x, " y=", regions(i)%y   
      end do
   end subroutine list_

end module omsourceregion_mod





